use std::{rc::Rc, sync::{Mutex, Arc}};

use libspa::ReadableDict;
use pipewire::{MainLoop, Context};
use regex::Regex;


fn deal_with_node(
    port: &pipewire::registry::GlobalObject<libspa::ForeignDict>,
    ids: AVec
) {
    
    let re = Regex::new(r"WEBRTC VoiceEngine").unwrap();

    if let Some(props) = &port.props {
        if let (Some(alias), Some(id)) = (props.get("port.alias"), props.get("node.id")) {
            if re.is_match(alias) {
                let mut a = ids.lock().ok().unwrap();
                a.push(id.to_string().parse::<u32>().unwrap());
                drop(a);
                change(true);
            }
        }
    }
}

type AVec = Arc<Mutex<Vec<u32>>>;

fn deal_remove_node(
    id: u32,
    ids: AVec
) {

    if ids.lock().ok().unwrap().len() == 0 {
        return;
    }


    let ids_clone_c = ids.lock().ok().unwrap().clone();
    let idsa: Vec<u32> = ids_clone_c.into_iter().filter(|&f| f != id).collect::<Vec<u32>>();

    if idsa.len() == 0 {
        change(false);
    }
    let mut ids_clone = ids.lock().ok().unwrap();
    *ids_clone = idsa;
    drop(ids_clone);
}

fn change(on: bool) {
    println!("change status {}", on);
    if on {
        let _resp = reqwest::blocking::get("http://aether:3000/start/stop");
    } else {
        let _resp = reqwest::blocking::get("http://aether:3000/stop/stop");
    }
}

fn main() -> Result<(), Box<dyn std::error::Error>> {
    println!("Hello, world!");

    let main_loop = MainLoop::new()?;
    let context = Context::new(&main_loop)?;
    let core = Rc::new(context.connect(None)?);
    
    let ids: AVec = Arc::new(Mutex::new(Vec::new()));
    let cpy = ids.clone();
    let registry = core.get_registry()?;

    let _listener = registry
        .add_listener_local()
        .global(move |global| match global.type_ {
            pipewire::types::ObjectType::Port => deal_with_node(global, cpy.clone()),
            _ => ()
        })
        .global_remove(move |id| deal_remove_node(id, ids.clone()))
       .register();

    main_loop.run();

    Ok(())
}
